

# Input data from our own flavour of HDF5-dataset

import biobabel
import numpy as np
import os
import datetime

from opensignalsreader import OpenSignalsReader



def load(fname):

    """ Load BioPAC acq file """
    
    bio = biobabel.Biodata() # create a new biodata object


    # Read OpenSignals file
    acq = OpenSignalsReader(fname)

    for k in acq.info:
        bio.add_meta(k,acq.info[k])
    
    SR = acq.sampling_rate

    bio.name = fname
    bio.date = acq.info['date']+' '+acq.info['time']

    for ch,label in zip(acq.channels,acq.labels):

        # ch is the channnel number, label is the corresponding label
        
        units = acq.units.get(label,'n/a')
        ran   = acq.ranges.get(label,'n/a')
        
        hdr = {
            'id'                :label,
            'participant'       :'participant',
            'sampling_frequency':SR,
            'modality'          :label,
            'units'             :units
        }
        dat = acq.signal(ch)
        bio.add_channel((hdr,np.array(dat)))
    
    return bio



from biobabel.io import load

from biobabel import *

from biobabel.biodata import Biodata

DATEFORMAT = "%Y/%m/%d %H:%M:%S %Z%z"


def display():
    # Launch ECG analysis
    import sys
    import os

    fname = None
    if len(sys.argv)>1:
        fname = sys.argv[1]
    else:

        filetypes = (
            ('HDF5 dataset', '*.hdf5'),
            ('All files', '*.*')
        )

        fname = fd.askopenfilename(
            title='Select your recording',
            initialdir='.',
            filetypes=filetypes)

    if not fname:
        print("You need to select a file. Exiting now.")
        sys.exit(-1)

    if not os.path.exists(fname):
        print("File {} does not seem to exist. Exiting now.".format(fname))
        sys.exit(-1)
        
    bio = load(fname)
    bio.print()
    bio.plot()




def guess_modality(nm):
    """ Guess the modality from the label of a signal column """
    l = nm.lower()
    for mod in ['ecg','eda','gsr','ppg']:
        if l.find(mod)>-1:
            return mod
    return nm
    




import biobabel
import numpy as np
import os
import datetime
import json


def load(fname):

    bio = biobabel.Biodata() # create a new biodata object
    bio.name = fname
    
    m_time = os.path.getmtime(fname)
    dt_m = datetime.datetime.fromtimestamp(m_time)
    bio.add_meta('date',dt_m.strftime(biobabel.DATEFORMAT))
   
    with open(fname,'r') as f:
        firstline = f.readline()
        #print(firstline)
        cont = f.read()

    # See if we can scrape more config data
    cfg = firstline.find('CONFIG')
    if cfg>-1:
        c = firstline[(cfg+6):]
        #print(c)
        config = json.loads(c)
        #print(config)
        for k in config:
            bio.add_meta(k,config[k])
        
    # Remove some initial nonsense data that typically enters these files
    startp = cont.find("# Start signal received")
    if startp<0:
        print("### ERROR, no starting point found!")

    contents = cont[startp:].split("\n")


    # Assumed unless ohterwise specified
    TIME_DIVISOR = 1000

    # TODO: could retrieve the field names from the data header itself

    alldata = []
    events  = []
    for ln in contents:
        if not ln: continue
        items = ln.split(' ')
        if items[1]=='#':
            ev = {
                'type':items[3],
                't':int(items[4])
            }
            events.append(ev)
            continue
        if items[1]=='0' and items[2]=='raw':
            # Try to parse it
            t, fsr, ecg, therm, ppg, snd = items[3:]
            alldata.append(
                {"t"    : int(t),
                 "fsr"  : int(fsr),
                 "ecg"  : int(ecg),
                 "therm": int(therm),
                 "ppg"  : int(ppg),
                 "snd"  : int(snd)} )

    t = [ d['t'] for d in alldata ]
    t0 = min(t)
    dt = np.diff(t)/TIME_DIVISOR

    mediandt = np.median(dt)
    
    print("-- A human may want to inspect this:")
    print("Time step values (in s) min={:.5f}, max={:.5f}, mean={:.5f}, median={:.5f}, SD={:.5f}".format(
        np.min(dt),np.max(dt),np.mean(dt),np.median(dt),np.std(dt)))
    THRESH = mediandt*1.1
    print("# of time steps >.{}: {}".format(THRESH,np.sum(dt>THRESH)))
    print("----")
    print()

    SR = 1/mediandt
            
    for chan in ['fsr','ecg','therm','ppg','snd']:
        hdr = {
            'id'                :chan,
            'participant'       :'participant',
            'sampling_frequency':SR,
            'modality'          :chan,
            'units'             :'a.u'
        }
        dat = [ d[chan] for d in alldata ]
        bio.add_channel((hdr,np.array(dat)))

        
    eventtypes = list(set([ x['type'] for x in events ]))
    markers = {}
    for e in eventtypes:
        markers[e]=[ (ev['t']-t0)/TIME_DIVISOR for ev in events if ev['type']==e ]
    bio.markers = markers
        
    return bio








# Input data from our own flavour of HDF5-dataset

import h5py
import numpy as np

import biobabel
import numpy as np
import os
import datetime



def load(fname):

    bio = biobabel.Biodata() # create a new biodata object

    hf = h5py.File(fname,'r')

    if 'name' in hf.attrs:
        bio.name = hf.attrs['name']
    else:
        bio.name = fname
    
    if 'date' in hf.attrs:
        bio.date = hf.attrs['date']    

    if 'participants' not in hf.attrs:
        print("# Error, missing participants attribute in the root.")
        
    participants = hf.attrs['participants']

    for p in participants:
        for ch in hf[p].keys():
            nm = "{}/{}".format(p,ch)
            dset = hf[p][ch]
            SR = dset.attrs['SR']
            mod = dset.attrs['modality']
            hdr = {'id'                :nm,
                   'participant'       :p,
                   'sampling_frequency':SR,
                   'modality'          :mod}
            dat = np.array(dset[:]) # convert into numpy array just to be sure
            bio.add_channel((hdr,dat))
            
    bio.markers = {}
    for m in hf.attrs.get('markers',[]):
        bio.markers[m] = hf.attrs[m][:] # get the markers in question, make a copy

    if 'meta' in hf: # if we have a meta dataset (should be empty but we use it to carry the attributes)
        #print("Scavenge meta")
        m = hf['meta']
        for k in m.attrs:
            #print("Add {}".format(k))
            bio.add_meta(k,m.attrs[k])

    return bio




import biobabel
import numpy as np
import os
import datetime


import pyxdf # pip3 install pyxdf


def load(fname):

    """ Load LSL format XDF file. """
    
    print("ATTENTION, this is a very case-specific script.")
    print("It might not work in general to read XDF files.")
    print("Make sure you know what you are doing if you proceed here.")

    
    bio = biobabel.Biodata() # create a new biodata object
    bio.name = fname

    # This file is included in bioread
    streams, header = pyxdf.load_xdf(fname)
    print(header)

    participants = [ s['info']['name'][0] for s in streams ]
    participants.sort()
    bio.participants = participants

    # Determine the common onset 
    ONSET_T, N_SAMP = get_onset(streams)

    
    for s in streams:
        info = s['info']
        print(" {} {}".format(info['name'][0],info['type'][0]))
        tp = info['type'][0].lower()
        #if tp!='ecg': continue # for now we only accept ECG signals

        modality = tp
        p = info['name'][0]

        t_sel = s['time_stamps']>ONSET_T
        rawdata = s["time_series"].T[0][t_sel] # just take the first stream, and only after the common starting point
        rawdata = rawdata[:N_SAMP] # take only the common chunk
        sz = rawdata.shape[0]
        assert sz==N_SAMP

        SR = float(info['nominal_srate'][0]) #info['effective_srate']
        units = info['desc'][0]['channels'][0]['channel'][0]['unit'][0]

        hdr = {'id'                :p,
               'participant'       :p,
               'sampling_frequency':SR,
               'modality'          :modality,
               'units'             :units
               }
        dat = rawdata
        bio.add_channel((hdr,np.array(dat)))


    bio.meta['date']=header['info']['datetime'][0]
    
    return bio



def get_onset(streams):
    """
    One issue is that in XDF, streams can start at different moments in time.
    Here we need to align them all to a common offset.
    The idea is to find the earliest moment in time where all streams have started,
    and then cut off anything in any stream before that.
    It throws away some data (downside) but allows us to have everything
    aligned in time (upside).
    """

    start_ts = [ min(s['time_stamps']) for s in streams ]
    ONSET_T = max(start_ts)

    n_samp = [ sum(s['time_stamps']>=ONSET_T) for s in streams ]
    print(n_samp)
    N_SAMP = min(n_samp) # take the smallest common time portion

    print("---------- TIMING ----------")
    print("Joint start t={}".format(ONSET_T))
    print("Stream onset deltas (should not be too great)")
    print([ t-ONSET_T for t in start_ts ])
    print("Largest common duration: {} samples".format(N_SAMP))
    print("Duration mismatches (proportion of common duration)")
    print(" ".join([ "{:.4f}".format(n/N_SAMP) for n in n_samp ]))
    print()

    print("---------- SAMPLING RATES ----------")
    nominal_sr = [ float(s['info']['nominal_srate'][0]) for s in streams ]
    eff_sr     = [ float(s['info']['effective_srate']) for s in streams ]
    print("Nominal sampling rates:")
    print(" ".join([ "{:.2f}".format(n) for n in nominal_sr ]))
    print("Effective sampling rates:")
    print(" ".join([ "{:.2f}".format(n) for n in eff_sr ]))
    print()

    return ONSET_T,N_SAMP




import numpy as np
import h5py
import time





class Biodata:
    """ 
    Represents a collection of streams of physiological data (channels)
    as well as some set of triggers (event markers).
    """

    
    def __init__(self):
        self.clear()

    def clear(self):
        """ Clear all data currently in the object """
        self.date         = ''
        self.channels     = []
        self.markers      = {}
        self.meta         = {} # miscellaneous metadata
        self.name         = ''


    def summarize_meta(self):
        # Pretty printing courtesy of
        # https://stackoverflow.com/questions/54573444/how-to-print-key-value-pairs-of-a-dict-as-an-aligned-table
        if len(self.meta.keys()):
            max_key_len = max(map(len,self.meta)) # find the maximum key length
            format_string = '· {{key:{}}}  {{value}}\n'.format(max_key_len)
            toret = ""
            for key, value in self.meta.items():
                toret += (format_string.format(key=key, value=value))
            return toret
        else:
            return ""
        #return '\n'.join([ "· {}={}".format(k,self.meta[k]) for k in self.meta ])
        
    def add_meta(self,k,v):
        self.meta[k]=v

    def uniquefy(self,ident):
        ids = self.find_channels()
        if ident not in ids:
            return ident
        cnt = 1
        newid = "{}.{}".format(ident,cnt)
        while newid in self.find_channels():
            cnt += 1
            newid = "{}.{}".format(ident,cnt)
        return newid

    def add_channel(self,hdrdat):
        """ Add the channel with specified header information and data """

        hdr,dat = hdrdat
        newid = hdr['id'].replace('/','_') # slashes don't work because HDF5 gets confused
        hdr['id']=self.uniquefy(newid) # make the ID unique in case it already exists
        self.channels.append((hdr,dat))
        
    def find_channels(self,crit={}):
        """ Find the channel id's satisfying certain criteria """
        chans = []
        for hdr,_ in self.channels:
            ok = True
            for k in crit:
                if hdr.get(k,None)!=crit[k]:
                    ok = False
            if ok:
                chans.append(hdr['id'])
        return chans


    def get_participants(self):
        part = []
        for hdr,_ in self.channels:
            p= hdr.get('participant',None)
            if p and p not in part:
                part.append(p)
        part.sort()
        return part

    def get(self,chid):
        # Get a particular channel data
        for hdr,dat in self.channels:
            if hdr.get('id',None)==chid:
                return hdr,dat
        print("Could not find channel {}".format(chid))
        return None,None

    def get_time(self,chid):
        # Given a channel, reproduce a time vector
        hdr,dat = self.get(chid)
        t = np.arange(dat.shape[0])/hdr['sampling_frequency']
        return t
    

    def get_duration(self,chid):
        """ Get the duration of a channel in seconds """
        hdr,dat  = self.get(chid)
        SR       = hdr['sampling_frequency']
        nsamp    = dat.shape[0]
        dur      = nsamp/SR
        return dur


    
    def summary(self):
        """ Return a summary of the current data (in str format) """
        ret = "Summary of {}\n".format(self.name)
        ret += self.summarize_meta()+"\n"
        if self.date:
            ret += '[ date : {} ]\n'.format(self.date)
        for p in self.get_participants():
            ret += "\nParticipant '{}'\n".format(p)
            chans = self.find_channels({"participant":p})
            for chan in chans:
                ret += "∟ {}\n".format(self.summarize_channel(chan))

        if len(self.markers):
            ret += "\nMarkers:\n"
            for m in self.markers:
                ret += "∟ marker {} : {} events\n".format(
                    m,
                    len(self.markers[m]),
                )
        return (ret)



    def summarize_channel(self,chid):
        """ Return a string summary of the channel indicated by the given id. """
        hdr,dat  = self.get(chid)
        SR       = hdr['sampling_frequency']
        nsamp    = dat.shape[0]
        dur      = nsamp/SR

        units = 'in {} '.format(hdr['units']) if 'units' in hdr else ''
        
        ret = "channel {} [ modality {} {}] {} samples @ {:.1f} Hz = {:.1f} s".format(
            chid,
            hdr.get('modality','N/A'),
            units,
            nsamp,
            SR,
            dur
        )

        return ret


    def print(self):
        summ = self.summary()
        print(summ)

    

    def plot(self):

        import matplotlib.pyplot as plt

        chans = self.find_channels()
        if not len(chans):
            print("No channels to plot.")
            return

        def get_colors(N):
            import colorsys
            HSV_tuples = [(x*1.0/N, 0.5, 0.5) for x in range(N)]
            RGB_tuples = map(lambda x: colorsys.hsv_to_rgb(*x), HSV_tuples)
            return list(RGB_tuples)

        COLORS = get_colors(len(chans))

        f,axs =plt.subplots(len(chans),1,sharex=True,figsize=(12,7),squeeze=False)

        for i,chan in enumerate(chans):
            ax = axs[i][0]
            hdr,vals = self.get(chan)
            t = self.get_time(chan)
            col = COLORS[i]
            ax.plot(t,vals,color=col)
            ax.set_title(chan)

        for m in self.get_markers():
            evs = self.get_marker(m)
            for t in evs:
                ax.axvline(x=t)

        if self.name:
            plt.suptitle(self.name)
        plt.tight_layout()
        plt.show()



        
        
    
    def get_markers(self):
        mrk= list(self.markers.keys())
        mrk.sort()
        return mrk

    def get_marker(self,m):
        return self.markers.get(m,[])



        
        
    def save(self,fname):
        """
        Save current data file in hdphysio5 format

        fname : filename of the file to be created
        """

        if not fname.lower().endswith('.hdf5'):
            print("Currently only saving in hdphysio5 format.")
            return

        # Go ahead and save
        
        ## Create the HDF5 version
        hf = h5py.File(fname, "w")
        participants = self.get_participants() # set participants attribute
        if self.name:
            hf.attrs['name']=self.name
        hf.attrs['participants']=participants
        if self.date:
            hf.attrs['date']=self.date
        else:
            hf.attrs['date']=time.strftime("%m/%d/%Y %H:%M:%S %Z%z")

        # Create an empty dataset to hold meta attributes
        m = hf.create_dataset("meta", data=h5py.Empty("f"))
        for k in self.meta:
            m.attrs[k]=self.meta[k]

        # Create the markers
        eventtypes = self.get_markers()
        if eventtypes:
            hf.attrs['markers']=eventtypes
            for e in eventtypes:
                evs = self.get_marker(e)
                hf.attrs[e]=evs

        # Add the data channels for each participant
        for p in participants:
            dat = hf.create_group(p)

            # Find channels for this participant
            chans = self.find_channels({'participant':p})
            for chan in chans:
                hdr,dat = self.get(chan)
                modality = hdr['modality']
                sz = dat.shape[0]
                dset = hf[p].create_dataset(chan,(sz,),dtype='f',data=dat)
                dset.attrs['SR']          =hdr['sampling_frequency']
                dset.attrs['participant'] =p
                dset.attrs['modality']    =modality
                dset.attrs['units']       =hdr.get('units','arbitrary')

        hf.close()




import biobabel
import numpy as np
import os
import datetime
import pandas as pd


#
#
#
# Module for loading generic CSV
#
#   Basically what we try to do is guess as much as possible.
#
#





def guess_time_column(dat):
    # Given a table with some columns, guess which one might be the time column
    # Return the name of the column, and time divider
    for col in dat.columns:

        # See if this could be a time column
        vals = dat[col]
        ds = np.diff(vals)

        # Check : values need to be increasing
        if not min(ds)>0:
            continue

        # Check : values need to increase by roughly the same amount each time
        cv = np.std(ds)/np.mean(ds)
        if cv>.1:
            continue

        # If we get this far it's looking pretty good!
        print("Guessed '{}' is the time column.".format(col))
        return col,1000 # assume ms as default sampling unit
        
    return None,1







def load(fname):

    """ Load a file with CSV format. """

    print("Generic CSV load")
    
    bio = biobabel.Biodata() # create a new biodata object
    bio.name = fname
    
    m_time = os.path.getmtime(fname)
    dt_m = datetime.datetime.fromtimestamp(m_time)
    bio.meta['date']=dt_m.strftime(biobabel.DATEFORMAT)

    ## gb['renames'] comes from the file configuration
    tab = pd.read_csv(fname,sep=None,engine='python') # pandas will try to guess the delimiter, this might fail!


    tcol,TIME_DIVIDER = guess_time_column(tab)
    if tcol:

        tab[tcol]=tab[tcol]/TIME_DIVIDER  # express in s
        tdur = max(tab[tcol])-min(tab[tcol])
        dt = np.diff(tab[tcol])
        mediandt=np.median(dt)
        print("-- A human may want to inspect this:")
        print("Time step values (in s) min={:.5f}, max={:.5f}, mean={:.5f}, median={:.5f}, SD={:.5f}".format(
            np.min(dt),np.max(dt),np.mean(dt),mediandt,np.std(dt)))
        THRESH = mediandt*1.1
        print("# of time steps >.{}: {}".format(THRESH,np.sum(dt>THRESH)))
        print("----")
        print()

    else:
        # Create some defaults
        TIME_DIVIDER = 1000 # assume time units by default are in ms
        tcol='__t__' # create a time column
        tab[tcol]=np.arange(tab.shape[0])
        mediandt=1

    SR = 1/mediandt

    participant = 'participant'

    for col in tab.columns:
        nm = col.lower()

        if nm=='t': continue

        hdr = {
            'id'                :col,
            'participant'       :participant,
            'sampling_frequency':SR,
            'modality'          :biobabel.guess_modality(nm),
            'units'             :'V'}
        data = 3.3 * (np.array(tab[col]) / 1023)
        bio.add_channel((hdr,data))

    return bio







import biobabel
import numpy as np
import os
import datetime


import pyedflib



def load(fname):

    """ Load EDF file. """
    
    bio = biobabel.Biodata() # create a new biodata object
    bio.name = fname

    h = pyedflib.highlevel.read_edf_header(fname)

    # Fill in time stamp
    m_time = os.path.getmtime(fname)
    dt_m = datetime.datetime.fromtimestamp(m_time)
    dt = h.get('startdate',dt_m)
    bio.meta['date']=dt.strftime(biobabel.DATEFORMAT)
    for k in h: # copy over the header information
        if k not in ['SignalHeaders','channels']:
            bio.add_meta(k,str(h[k]))

    # Get participant identifier
    participant = h.get('patientcode','participant')

    # Get signal headers
    headers = h['SignalHeaders']    

    # Start reading data
    f = pyedflib.EdfReader(fname)
    n = f.signals_in_file
    signal_labels = f.getSignalLabels()
    
    assert n==len(headers)
    
    for i,head in enumerate(headers):

        SR = head['sample_rate']
        lab = signal_labels[i]
        hdr = {
            'id'                :lab,
            'participant'       :participant,
            'sampling_frequency':SR,
            'modality'          :biobabel.guess_modality(lab),
            'units'             :head.get('dimension','unknown')
        }
        dat = f.readSignal(i)
        bio.add_channel((hdr,np.array(dat)))
    
    return bio






# Input data from our own flavour of HDF5-dataset

import biobabel
import numpy as np
import os
import datetime

try:
    from pybdf import bdfRecording
except ImportError as e:
    bdfRecording = None # on some platform this package seemed to give issues



def load(fname):

    """ Load BDF file """

    if not bdfRecording:
        raise ImportError(f"Loading BDF files requires pybdf, which failed to import.")
    
    bio = biobabel.Biodata() # create a new biodata object

    # This file is included in bioread
    bdfRec = bdfRecording(fname)
    
    bio.name = fname
    m_time   = os.path.getmtime(fname)
    dt_m     = datetime.datetime.fromtimestamp(m_time)
    bio.date = dt_m.strftime(biobabel.DATEFORMAT)

    rec = bdfRec.getData()
    
    for i,ch in enumerate(rec['chanLabels']):
    
        hdr = {
            'id'                :ch,
            'participant'       :'participant',
            'sampling_frequency':bdfRec.sampRate[i],
            'modality'          :ch.lower(),
            'units'             :ch.lower()
        }
        dat = rec['data'][i]
        bio.add_channel((hdr,np.array(dat)))

    # TODO: process event markers
    #for m in data.event_markers:
    #    print('unprocessed marker {0}: Channel {1}, type {2}'.format(m.text, m.channel_name, m.type))

    #bio.markers = {}
    #for m in hf.attrs.get('markers',[]):
    #    bio.markers[m] = hf.attrs[m][:] # get the markers in question, make a copy

    return bio




import biobabel.load_teensyecg
import biobabel.load_hdphysio5
import biobabel.load_lsl
import biobabel.load_acq
import biobabel.load_opensignals
import biobabel.load_bramsbiobox
import biobabel.load_genericcsv
import biobabel.load_bdf
import biobabel.load_edf

import os


def load(fname,dialect=None):
    """
    Load physiology signal file.

    fname : filename of the file to be read
    dialect : the file format. If None, the format is guessed
    """
    
    if not os.path.exists(fname):
        print("File '{}' does not seem to exist.".format(fname))
        return
    
    if dialect==None:
        # Try to guess
        dialect = guess_dialect(fname)
        if dialect:
            print("Guessed that this is {} format.".format(dialect))
        
    if dialect=="teensyecg":
        return biobabel.load_teensyecg.load(fname)

    if dialect=="hdphysio5":
        return biobabel.load_hdphysio5.load(fname)

    if dialect=="lsl":
        return biobabel.load_lsl.load(fname)

    if dialect=="acq":
        return biobabel.load_acq.load(fname)

    if dialect=="opensignals":
        return biobabel.load_opensignals.load(fname)

    if dialect=="csv":
        return biobabel.load_genericcsv.load(fname)

    if dialect=="bdf":
        return biobabel.load_bdf.load(fname)

    if dialect=="edf":
        return biobabel.load_edf.load(fname)
    
    if dialect=="bramsbiobox":
        return biobabel.load_bramsbiobox.load(fname)
    
    return None # did not manage to load





def guess_dialect(fname):

    if fname.lower().endswith('.bdf'):
        return "bdf" # guess
    if fname.lower().endswith('.hdf5'):
        return "hdphysio5" # guess
    if fname.lower().endswith('.xdf'):
        return "lsl" # guess
    if fname.lower().endswith('.edf'):
        return "edf" # guess
    if fname.lower().endswith('.acq'):
        return "acq" # guess

    if fname.lower().endswith('.txt'):

        # To guess the dialect, we have to actually probe the file itself
        with open(fname) as f:
            ln = f.readline()

        if ln.find('OpenSignals')>-1:
            return 'opensignals'
  
        return "csv" # probably best to try


    if fname.lower().endswith('.csv'):

        # To guess the dialect, we have to actually probe the file itself
        with open(fname) as f:
            ln = f.readline().lower()

        if ln.find('time(ms)')>-1 and ln.find('xaccel')>-1 and ln.find('gauge')>-1:
            return 'bramsbiobox'

        else:
            return 'csv' # generic CSV

    return 'csv' # default is generic CSV




import biobabel
import numpy as np
import os
import datetime
import pandas as pd


col_info = {
    "Time(ms)": "t",
    "PPG": "ppg",
    "ECG": "ecg",
    "Gauge": "gauge",
    "Thermistor": "therm",
    "Xaccel": "xaccel",
    "Yaccel": "yaccel",
    "Zaccel": "zaccel"
}



def load(fname):

    """ Load a file with the BRAMS-Bio-Box format. """
    
    bio = biobabel.Biodata() # create a new biodata object
    bio.name = fname
    
    m_time = os.path.getmtime(fname)
    dt_m = datetime.datetime.fromtimestamp(m_time)
    bio.meta['date']=dt_m.strftime(biobabel.DATEFORMAT)

    ## gb['renames'] comes from the file configuration
    tab = pd.read_csv(fname,sep=',')

    TIME_DIVIDER = 1000

    tcol = 'Time(ms)'
    tab[tcol]=tab[tcol]/TIME_DIVIDER  # express in s

    tdur = max(tab[tcol])-min(tab[tcol])
    dt = np.diff(tab[tcol])
    mediandt=np.median(dt)
    print("-- A human may want to inspect this:")
    print("Time step values (in s) min={:.5f}, max={:.5f}, mean={:.5f}, median={:.5f}, SD={:.5f}".format(
        np.min(dt),np.max(dt),np.mean(dt),mediandt,np.std(dt)))
    THRESH = mediandt*1.1
    print("# of time steps >.{}: {}".format(THRESH,np.sum(dt>THRESH)))
    print("----")
    print()

    SR = 1/mediandt

    participant = 'participant'

    for col in tab.columns:
        nm = col
        if col in col_info: nm=col_info[nm]

        if nm=='t': continue

        hdr = {
            'id'                :col,
            'participant'       :participant,
            'sampling_frequency':SR,
            'modality'          :nm,
            'units'             :'V'}
        data = 3.3 * (np.array(tab[col]) / 1023)
        bio.add_channel((hdr,data))

    return bio








# Input data from our own flavour of HDF5-dataset

import biobabel
import numpy as np
import os
import datetime

import bioread


def load(fname):

    """ Load BioPAC acq file """
    
    bio = biobabel.Biodata() # create a new biodata object

    # This file is included in bioread
    data = bioread.read_file(fname)
    
    bio.name = fname
    earliest = data.earliest_marker_created_at
    if earliest:
        dt       = earliest.strftime(biobabel.DATEFORMAT)#"%m/%d/%Y %H:%M:%S %Z%z")
        bio.date = dt
    else:
        m_time = os.path.getmtime(fname)
        dt_m = datetime.datetime.fromtimestamp(m_time)
        bio.meta['date']=dt_m.strftime(biobabel.DATEFORMAT)
        

    for ch in data.channels:

        print(" {}".format(ch.name))

        hdr = {
            'id'                :ch.name,
            'participant'       :'participant',
            'sampling_frequency':ch.samples_per_second,
            'modality'          :ch.name.split(' ')[0].lower(),
            'units'             :ch.units
        }
        dat = ch.data
        bio.add_channel((hdr,np.array(dat)))
    
    bio.markers = {}
    for m in data.event_markers:
        tp = str(m.type)
        idx = m.sample_index
        t = idx/ch.samples_per_second
        bio.markers[tp] = bio.markers.get(tp,[]) + [t] # Append the marker

    #bio.markers = {}
    #for m in hf.attrs.get('markers',[]):
    #    bio.markers[m] = hf.attrs[m][:] # get the markers in question, make a copy

    return bio
